# -*- coding: utf8 -*-
#
# XSL - graphical interface for SL
# Copyright (C) 2007-2016 Devaev Maxim
#
# This file is part of XSL.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


import Qt
import Config
import Const
import LangsList
import TextBrowser


#####
MyIcon = Config.DataRootDir+"/xsl/icons/xsl_16.png"
WaitPicture = Config.DataRootDir+"/xsl/pictures/circular.gif"

AllDictsDir = Config.DataRootDir+"/sl/dicts/"
IconsDir = Config.DataRootDir+"/xsl/icons/"


#####
def tr(str) :
	return Qt.QApplication.translate("@default", str)


#####
class DictInfoWindow(Qt.QDialog) :
	def __init__(self, dict_name, dict_info = None, parent = None) :
		Qt.QDialog.__init__(self, parent)

		self.setWindowTitle(tr("Dict Information"))
		self.setWindowIcon(Qt.QIcon(MyIcon))

		self.setMinimumSize(550, 400)
		self.resize(550, 400)

		#####

		self.main_layout = Qt.QVBoxLayout()
		self.main_layout.setContentsMargins(0, 0, 0, 0)
		self.main_layout.setSpacing(0)
		self.setLayout(self.main_layout)

		self.dict_info_browser_layout = Qt.QVBoxLayout()
		self.dict_info_browser_layout.setContentsMargins(0, 0, 0, 0)
		self.dict_info_browser_layout.setSpacing(0)
		self.main_layout.addLayout(self.dict_info_browser_layout)

		self.control_buttons_layout = Qt.QHBoxLayout()
		self.control_buttons_layout.setContentsMargins(6, 6, 6, 6)
		self.control_buttons_layout.setSpacing(6)
		self.main_layout.addLayout(self.control_buttons_layout)

		#####

		self.dict_name = Qt.QString(dict_name)

		self.is_loaded_flag = False

		self.dict_name_regexp = Qt.QRegExp("([^\\.]+)\\.((..)-(..))")

		#####

		self.dict_info_browser = TextBrowser.TextBrowser()
		if dict_info != None :
			self.dict_info_browser.setText(dict_info)
			self.is_loaded_flag = True
		self.dict_info_browser_layout.addWidget(self.dict_info_browser)

		self.wait_picture_movie = Qt.QMovie(WaitPicture)
		icon_width = icon_height = self.style().pixelMetric(Qt.QStyle.PM_SmallIconSize)
		self.wait_picture_movie.setScaledSize(Qt.QSize(icon_width, icon_height))
		self.wait_picture_movie.jumpToFrame(0)
		self.wait_picture_movie_label = Qt.QLabel()
		self.wait_picture_movie_label.setMovie(self.wait_picture_movie)
		self.wait_picture_movie_label.hide()
		self.control_buttons_layout.addWidget(self.wait_picture_movie_label)

		self.wait_message_label = Qt.QLabel(tr("Please wait..."))
		self.wait_message_label.hide()
		self.control_buttons_layout.addWidget(self.wait_message_label)

		self.control_buttons_layout.addStretch()

		self.update_info_button = Qt.QPushButton(Qt.QIcon(IconsDir+"update_16.png"), tr("&Update"))
		self.control_buttons_layout.addWidget(self.update_info_button)

		self.ok_button = Qt.QPushButton(Qt.QIcon(IconsDir+"ok_16.png"), tr("&OK"))
		self.ok_button.setDefault(True)
		self.control_buttons_layout.addWidget(self.ok_button)

		#####

		self.connect(self.update_info_button, Qt.SIGNAL("clicked()"), self.updateInfo)
		self.connect(self.ok_button, Qt.SIGNAL("clicked()"), self.accept)


	### Public ###

	def updateInfo(self) :
		if self.dict_name.isEmpty() :
			return

		self.update_info_button.blockSignals(True)
		self.update_info_button.setEnabled(False)

		self.wait_picture_movie_label.show()
		self.wait_picture_movie.start()
		self.wait_message_label.show()

		###

		dict_info = Qt.QString()
		dict_info.append(tr("<font class=\"text_label_font\">Caption</font>: %2<hr>").arg(self.dictCaption()))
		dict_info.append(tr("<font class=\"text_label_font\">Translate direction</font>: %2<hr>").arg(self.dictDirection()))
		dict_info.append(tr("<font class=\"text_label_font\">File path</font>: %2<hr>").arg(self.dictFilePath()))
		dict_info.append(tr("<font class=\"text_label_font\">File size (KB)</font>: %2<hr>").arg(self.dictFileSize()))
		self.dict_info_browser.setText(dict_info)

		dict_description, word_count = self.dictDescriptionAndWordCount()
		dict_info.append(tr("<font class=\"text_label_font\">Count of words</font>: %2<hr>").arg(word_count))
		dict_info.append(tr("<font class=\"text_label_font\">Description</font>: %2").arg(dict_description))
		self.dict_info_browser.setText(dict_info)

		###

		Qt.QCoreApplication.processEvents()

		self.wait_picture_movie_label.hide()
		self.wait_picture_movie.stop()
		self.wait_picture_movie.jumpToFrame(0)
		self.wait_message_label.hide()

		self.update_info_button.setEnabled(True)
		self.update_info_button.blockSignals(False)

	###

	def dictInfo(self) :
		return self.dict_info_browser.text()

	###

	def show(self) :
		Qt.QDialog.show(self)
		self.raise_()
		self.activateWindow()

		if not self.is_loaded_flag :
			self.updateInfo()
			self.is_loaded_flag = True


	### Private ###

	def dictCaption(self) :
		if self.dict_name_regexp.exactMatch(self.dict_name) :
			dict_caption = self.dict_name_regexp.cap(1)
		else :
			dict_caption = Qt.QString(self.dict_name)
		dict_caption.replace("_", " ")
		dict_caption.replace(".", " ")
		return dict_caption

	def dictDirection(self) :
		if self.dict_name_regexp.exactMatch(self.dict_name) :
			icon_width = icon_height = self.style().pixelMetric(Qt.QStyle.PM_SmallIconSize)
			return ( Qt.QString("<img src=\"%3\" width=\"%1\" height=\"%2\"> &#187; <img src=\"%4\" width=\"%1\" height=\"%2\">"
				"&nbsp;&nbsp;&nbsp;%5 &#187; %6 (%7)").arg(icon_width).arg(icon_height)
				.arg(IconsDir+"flags/"+self.dict_name_regexp.cap(3)+".png").arg(IconsDir+"flags/"+self.dict_name_regexp.cap(4)+".png")
				.arg(LangsList.langName(self.dict_name_regexp.cap(3))).arg(LangsList.langName(self.dict_name_regexp.cap(4)))
				.arg(self.dict_name_regexp.cap(2)) )
		else :
			return tr("Unavailable")

	def dictFilePath(self) :
		dict_file_path = Qt.QString(AllDictsDir+self.dict_name)
		return ( dict_file_path if Qt.QFile.exists(dict_file_path) else tr("Unavailable") )

	def dictFileSize(self) :
		dict_file = Qt.QFile(AllDictsDir+self.dict_name)
		return Qt.QString().setNum(dict_file.size() / 1024) # KB

	def dictDescriptionAndWordCount(self) :
		dict_description = Qt.QString()
		word_count = 0

		dict_file = Qt.QFile(AllDictsDir+self.dict_name)
		dict_file_stream = Qt.QTextStream(dict_file)

		if dict_file.open(Qt.QIODevice.ReadOnly) :
			first_comment_flag = False
			while not dict_file_stream.atEnd() :
				Qt.QCoreApplication.processEvents(Qt.QEventLoop.ExcludeUserInputEvents)
				str = dict_file_stream.readLine()
				if str.isEmpty() :
					continue
				if str[0] == "#" :
					str.remove(0, 1)
					str.remove("\n")
					str.append("<br>")
					dict_description.append(str)
					continue
				if str.contains("  ") :
					word_count += 1
			dict_file.close()
			dict_description.trimmed()

		if dict_description.isEmpty() :
			dict_description = tr("Unavailable")

		return dict_description, Qt.QString().setNum(word_count)

